/****************************************************************************

 Module
  global.c
  
 Description
  Variables of global interest in oAESis.
  
 Author(s)
 	cg (Christer Gustavsson <d2cg@dtek.chalmers.se>)

 Revision history
 
  960103 cg
   Added standard header.
 
 Copyright notice
  The copyright to the program code herein belongs to the authors. It may
  be freely duplicated and distributed without fee, but not charged for.
 
 ****************************************************************************/

/****************************************************************************
 * Used interfaces                                                          *
 ****************************************************************************/

#include <osbind.h>
#include <stdio.h>
#include <unistd.h>

#include "boot.h"
#include "debug.h"
#include "global.h"
#include "lxgemdos.h"
#include "types.h"
#include "vdi.h"

/****************************************************************************
 * Global variables                                                         *
 ****************************************************************************/

GLOBALVARS	globals;

/****************************************************************************
 * Module global variables                                                  *
 ****************************************************************************/

static WORD	open_physical_ws = 0; /* Set this to 1 if you want to
                                     start oAESis without running
                                     GEM. */

/****************************************************************************
 * Public functions                                                         *
 ****************************************************************************/

void	init_global(WORD physical) {
	WORD work_in[] = {1,7,1,1,1,1,1,1,1,1,2};
	WORD work_out[57];
	WORD dum;

	globals.rez = 1;

	Boot_parse_cnf();

	open_physical_ws = physical;

	if(open_physical_ws) {	
		/* open up a vdi workstation to use in the process */
	
		work_in[0] = globals.rez;
	
		Vdi_v_opnwk(work_in,&globals.vid,work_out);
	}
	else {
		WORD	dum;
		
		appl_init();
		globals.vid = graf_handle(&dum,&dum,&dum,&dum);
		appl_exit();
	};
	
	Vdi_vq_extnd(globals.vid,0,work_out);
	
	globals.screen.x = 0;
	globals.screen.y = 0;
	globals.screen.width = work_out[0] + 1;
	globals.screen.height = work_out[1] + 1;

	globals.num_pens = work_out[13];
	
  /* setup systemfont information */
  
  if(globals.screen.height >= 400) {
		globals.fnt_small_id = 1;
	 	globals.fnt_small_sz = 4;
	 	globals.fnt_regul_id = 1;
 		globals.fnt_regul_sz = 13;
 	}
 	else {
		globals.fnt_small_id = 1;
	 	globals.fnt_small_sz = 4;
	 	globals.fnt_regul_id = 1;
 		globals.fnt_regul_sz = 9;
 	};
 	 
 	Vdi_vst_font(globals.vid, globals.fnt_regul_id);
 	Vdi_vst_point(globals.vid,globals.fnt_regul_sz,&dum,&dum,&dum,&dum);

	globals.realmove = 0;

	globals.ap_pri = NULL;
	globals.applmenu = NULL;
	globals.accmenu = NULL;

  globals.win_vis = NULL;
	
	globals.mouse_x = 0;
	globals.mouse_y = 0;
	globals.mouse_button = 0;
	
	globals.arrowrepeat = 200;
	
	Vdi_vqt_attributes(globals.vid,work_out);
	
	globals.blwidth = work_out[8] + 3;
	globals.blheight = work_out[9] + 3;
	globals.clwidth = work_out[8];
	globals.clheight = work_out[9];

	globals.time = 0L;
}

void	exit_global(void) {
	if(open_physical_ws) {
		Vdi_v_clswk(globals.vid);
	};
}
