/****************************************************************************

 Module
  wind.c
  
 Description
  Window routines used in oAESis.
  
 Author(s)
 	cg     (Christer Gustavsson <d2cg@dtek.chalmers.se>)

 Revision history
 
  951226 cg
   Added standard header.
  960101 cg
   Added BEG_MCTRL and END_MCTRL modes to wind_update.
  960102 cg
   WF_TOP mode of wind_get() implemented.
  960103 cg
   WF_NEWDESK mode of wind_set() implemented.
   WF_HSLIDE, WF_VSLIDE, WF_VSLSIZE and WF_HSLSIZE modes of wind_set()
   and wind_get() implemented.
 
 Copyright notice
  The copyright to the program code herein belongs to the authors. It may
  be freely duplicated and distributed without fee, but not charged for.
 
 ****************************************************************************/

/****************************************************************************
 * Used interfaces                                                          *
 ****************************************************************************/

#define	__TCC_COMPAT__

#include	<aesbind.h>
#include	<alloc.h>
#include	<basepage.h>
#include	<mintbind.h>
#include	<osbind.h>
#include	<signal.h>
#include	<stdio.h>
#include	<string.h>
#include	<support.h>

#include "appl.h"
#include "debug.h"
#include "evnt.h"
#include "evnthndl.h"
#include "global.h"
#include "lxgemdos.h"
#include "memdefs.h"
#include "mesagdef.h"
#include "misc.h"
#include "objc.h"
#include "rdwrs.h"
#include "resource.h"
#include "types.h"
#include "vdi.h"
#include "wind.h"

/****************************************************************************
 * Macros                                                                   *
 ****************************************************************************/

#define	WIN_NR	10
#define DRAW_SEM 0x6f445257L    /*'oDRW'*/

#define W_ALL (-1)  /* set _all_ widgets */

/****************************************************************************
 * Typedefs of module global interest                                       *
 ****************************************************************************/

typedef struct {
  WORD wid;
  RECT r;
}WMSG;

/****************************************************************************
 * Module global variables                                                  *
 ****************************************************************************/

static CSEMA update = {
	DRAW_SEM,
	-1,
	0,
	-1
};

static WORD	elemnumber = -1;
static WORD	tednumber;


static WINLIST *win_list[WIN_NR];
static WINLIST *win_free = 0L;

static WORD drawque[2];

static WORD	win_next = 0;

static WORD drawid;

static OBJC_COLORWORD top_colour[20] = {
	BLACK,BLACK,1,7,LWHITE,
	BLACK,BLACK,1,7,LWHITE,
	BLACK,BLACK,1,7,LWHITE,
	BLACK,BLACK,1,7,LWHITE,
	BLACK,BLACK,1,7,LWHITE,
	BLACK,BLACK,1,7,LWHITE,
	BLACK,BLACK,1,7,LWHITE,
	BLACK,BLACK,1,7,LWHITE,
	BLACK,BLACK,1,7,LWHITE,
	BLACK,BLACK,1,7,LWHITE,
	BLACK,BLACK,1,7,LWHITE,
	BLACK,BLACK,1,7,LWHITE,
	BLACK,BLACK,1,7,LWHITE,
	BLACK,BLACK,1,7,LWHITE,
	BLACK,BLACK,1,7,LWHITE,
	BLACK,BLACK,1,7,LWHITE,
	BLACK,BLACK,1,7,LWHITE,
	BLACK,BLACK,1,7,LWHITE,
	BLACK,BLACK,1,7,LWHITE,
	BLACK,BLACK,1,7,LWHITE
};

static OBJC_COLORWORD untop_colour[20] = {
	BLACK,BLACK,1,7,LWHITE,
	BLACK,BLACK,1,7,LWHITE,
	BLACK,BLACK,1,7,LWHITE,
	BLACK,BLACK,1,7,LWHITE,
	BLACK,BLACK,1,7,LWHITE,
	BLACK,BLACK,1,7,WHITE,
	BLACK,BLACK,1,7,LWHITE,
	BLACK,BLACK,1,7,LWHITE,
	BLACK,BLACK,1,7,LWHITE,
	BLACK,BLACK,1,7,LWHITE,
	BLACK,BLACK,1,7,LWHITE,
	BLACK,BLACK,1,7,LWHITE,
	BLACK,BLACK,1,7,LWHITE,
	BLACK,BLACK,1,7,LWHITE,
	BLACK,BLACK,1,7,LWHITE,
	BLACK,BLACK,1,7,LWHITE,
	BLACK,BLACK,1,7,LWHITE,
	BLACK,BLACK,1,7,LWHITE,
	BLACK,BLACK,1,7,LWHITE,
	BLACK,BLACK,1,7,LWHITE
};

static WORD widgetmap[] = {
	0,0,WCLOSER,WMOVER,WFULLER,
	WINFO,0,0,WSIZER,0,
	WUP,WDOWN,WVSLIDER,0,0,
	WLEFT,WRIGHT,WHSLIDER,0,0
};

/****************************************************************************
 * Local functions (use static!)                                            *
 ****************************************************************************/

static drawproc(LONG arg) {
  WMSG	m;
  WINSTRUCT	*ws;
  WORD	work_in[] = {1,7,1,1,1,1,1,1,1,1,2};
  WORD	work_out[57];
  WORD	vid;

	NOT_USED(arg);
  
  Fclose(drawque[1]);

  /* open up a vdi workstation to use in the process */
  
  Vdi_v_opnvwk(work_in,&vid,work_out);
  
  while(1) {		
		Fread(drawque[0],sizeof(WMSG),&m);

		Wind_beg_update(0);
		Rdwrs_operation(WSTARTREAD);
    
    ws = findwin(m.wid);
    
    if(ws) {
    	if(globals.realmove) {
				m.r.x += ws->totsize.x;
				m.r.y += ws->totsize.y;
			};

			drawwinelem(vid,ws,&m.r,0);
    };
		
		Rdwrs_operation(WENDREAD);
		Wind_end_update(0);
	};
}

static void set_widget_colour(WINSTRUCT *win,WORD widget,OBJC_COLORWORD *untop,OBJC_COLORWORD *top) {
	U_OB_SPEC      *ob_spec;
	WORD           object = 0;
	OBJC_COLORWORD *colour;

	if(win->tree) {
		object = widgetmap[widget];
		
		if(win->tree[object].ob_flags & INDIRECT) {
			ob_spec = (U_OB_SPEC *)win->tree[object].ob_spec.indirect;
		}
		else {
			ob_spec = (U_OB_SPEC *)&win->tree[object].ob_spec;
		};
		
		switch(win->tree[object].ob_type & 0xff) {
		case G_BOX:
		case G_IBOX:
		case G_BOXCHAR:
			colour = &((OBJC_COLORWORD *)ob_spec)[1];
			break;
		
		case G_TEXT:
		case G_BOXTEXT:
		case G_FTEXT:
		case G_FBOXTEXT:
			colour = (OBJC_COLORWORD *)&ob_spec->tedinfo->te_color;
			break;
		
		case G_IMAGE:
			colour = (OBJC_COLORWORD *)&ob_spec->bitblk->bi_color;
			break;
		
		case G_BUTTON:
		case G_PROGDEF:
		case G_STRING:
		case G_TITLE:
			return;
			
		default:
			DB_printf("Unsupported type %d in set_widget_colour.",win->tree[object].ob_type);
		};
		
		if(win->status & WIN_ISTOPPED) {
			*colour = *top;
		}
		else {
			*colour = *untop;
		};
	};
}

/*crwinelem creates an object containing the elements given in elem.
  The elements are equal to those given to wind_create. An object
  created with crwinelem should be deleted with delwinelem.*/
  
static OBJECT	*crwinelem(WORD elem) {
	WORD	bottomsize = 0;
	WORD	headsize = 0;
	WORD	topsize = 0,tnr = 0;
	WORD	rightsize = 0;
	
	WORD	i = 0;
	
	LONG	size;
	
	OBJECT	*t;
	
	TEDINFO	*ti;
	
	while(elemnumber == -1) {
		switch(windowtad[i].ob_type) {
			case	G_TEXT		:
			case	G_BOXTEXT	:
			case	G_FTEXT		:
			case	G_FBOXTEXT	:
				tnr++;
		}
		
		if(windowtad[i].ob_flags & LASTOB)
		{
			elemnumber = i + 1;
			tednumber = tnr;
		}
		
		i++;
	}
	
	size = sizeof(OBJECT) * elemnumber + sizeof(TEDINFO) * tednumber;
	
	t = (OBJECT *)Mxalloc(size,GLOBALMEM);
	
	ti = (TEDINFO *)&t[elemnumber];
	
	memcpy(t,windowtad,sizeof(OBJECT) * elemnumber);
	
	for(i = 0; i < elemnumber; i++) {
		switch(windowtad[i].ob_type) {
			case	G_TEXT		:
			case	G_BOXTEXT	:
			case	G_FTEXT		:
			case	G_FBOXTEXT	:
				t[i].ob_spec.tedinfo = ti;
				memcpy(ti, windowtad[i].ob_spec.tedinfo, sizeof(TEDINFO));
				ti++;
		};
	};
	
	if((HSLIDE | LFARROW | RTARROW) & elem) {
		bottomsize = t[WLEFT].ob_height + (D3DSIZE << 1);
	};
	
	if((CLOSER | MOVER | FULLER) & elem) {
		topsize = t[WMOVER].ob_height + (D3DSIZE << 1);
	};
	
	if(INFO & elem) {
		headsize = topsize + t[WINFO].ob_height - 1;
	}
	else {
		headsize = topsize;
	};
	
	if((LFARROW | HSLIDE | RTARROW) & elem) {
		bottomsize = t[WLEFT].ob_height + (D3DSIZE << 1);
	};
	
	if(((bottomsize == 0) && (SIZER & elem))
		|| ((VSLIDE | UPARROW | DNARROW) & elem)) {
		rightsize = t[WSIZER].ob_width + (D3DSIZE << 1);
	};
	
	if(CLOSER & elem) {
		t[WCLOSER].ob_flags &= ~HIDETREE;	
	}	
	else {
		t[WCLOSER].ob_flags |= HIDETREE;
	}
	
	if(FULLER & elem) {
		t[WFULLER].ob_flags &= ~HIDETREE;	
	}	
	else {
		t[WFULLER].ob_flags |= HIDETREE;
	}
	
	if(SIZER & elem) {
		t[WSIZER].ob_flags &= ~HIDETREE;
		t[SFILLOUT].ob_flags |= HIDETREE;	
	}	
	else {
		t[WSIZER].ob_flags |= HIDETREE;
		
		if((bottomsize > 0) && (rightsize > 0)) {
			t[SFILLOUT].ob_flags &= ~HIDETREE;
			t[SFILLOUT].ob_x = t[WSIZER].ob_x;
			t[SFILLOUT].ob_y = t[WSIZER].ob_y;
		}
		else {
			t[SFILLOUT].ob_flags |= HIDETREE;
		}
	}
	
	if(INFO & elem) {
		t[WINFO].ob_flags &= ~HIDETREE;
		
		t[WINFO].ob_width = t[WINDOW].ob_width;
		
		if(topsize == 0) {
			t[WINFO].ob_y = 0;
		}
	}
	else {
		t[WINFO].ob_flags |= HIDETREE;
	}
	
	if(MOVER & elem) {
		t[WMOVER].ob_flags &= ~HIDETREE;
		
		if(CLOSER & elem) {
			t[WMOVER].ob_x = t[WCLOSER].ob_width + (D3DSIZE << 1);
		}
		else {
			t[WMOVER].ob_x = D3DSIZE;
		};
		
		if(FULLER & elem) {
			t[WMOVER].ob_width = t[WFULLER].ob_x - t[WMOVER].ob_x - (D3DSIZE << 1) + 1;
		}
		else {
			t[WMOVER].ob_width = t[WINDOW].ob_width - t[WMOVER].ob_x - D3DSIZE;
		};
	}
	else {
		t[WMOVER].ob_flags |= HIDETREE;
	};
	
	if(!(elem & MOVER) && (topsize > 0) && ((headsize > topsize) 
		|| (!(elem & INFO)))) {
		t[TFILLOUT].ob_x = D3DSIZE;
		t[TFILLOUT].ob_flags &= ~HIDETREE;
		
		if(CLOSER & elem) {
			t[TFILLOUT].ob_x = t[WCLOSER].ob_width + (D3DSIZE * 3);
		}
		else {
			t[TFILLOUT].ob_x = D3DSIZE;
		};
		
		if(FULLER & elem) {
			t[TFILLOUT].ob_width = t[WFULLER].ob_x - t[TFILLOUT].ob_x - D3DSIZE;
		}
		else {
			t[TFILLOUT].ob_width = t[WINDOW].ob_width - t[TFILLOUT].ob_x - D3DSIZE;
		};
	}
	else {
		t[TFILLOUT].ob_flags |= HIDETREE;
	};
	
	if(elem & UPARROW) {
		t[WUP].ob_flags &= ~HIDETREE;
		t[WUP].ob_y = headsize + D3DSIZE - 1;
	}
	else {
		t[WUP].ob_flags |= HIDETREE;
	};

	if(elem & DNARROW) {
		t[WDOWN].ob_flags &= ~HIDETREE;
		
		if((bottomsize == 0) && (SIZER & elem)) {
			t[WDOWN].ob_y = t[WINDOW].ob_height - t[WDOWN].ob_height
				- t[WSIZER].ob_height - (3*D3DSIZE) + 1;
		}
		else {
			t[WDOWN].ob_y = t[WINDOW].ob_height - t[WDOWN].ob_height
				- bottomsize - D3DSIZE + 1;
		};
	}
	else {
		t[WDOWN].ob_flags |= HIDETREE;
	};
	
	if(elem & VSLIDE) {
		t[WVSB].ob_flags &= ~HIDETREE;
		
		if(UPARROW & elem) {
			t[WVSB].ob_y = t[WUP].ob_y + t[WUP].ob_height + (D3DSIZE << 1) - 1;
		}
		else {
			t[WVSB].ob_y = headsize + D3DSIZE;
		}
		
		if(DNARROW & elem) {
			t[WVSB].ob_height = t[WDOWN].ob_y - t[WVSB].ob_y - (D3DSIZE << 1) + 1;
		}
		else {
			if(SIZER & elem) {
				t[WVSB].ob_height = t[WSIZER].ob_y - t[WVSB].ob_y;
			}
			else {
				t[WVSB].ob_height = t[WINDOW].ob_height - t[WVSB].ob_y
					- bottomsize;
			}
		}
	}
	else
	{
		t[WVSB].ob_flags |= HIDETREE;
	}
	
	if(!(VSLIDE & elem) && (rightsize > 0))
	{
		t[RFILLOUT].ob_flags &= ~HIDETREE;
		
		if(UPARROW & elem)
		{
			t[RFILLOUT].ob_y = t[WUP].ob_y + t[WUP].ob_height + (D3DSIZE << 1) - 1;
		}
		else
		{
			t[RFILLOUT].ob_y = headsize + D3DSIZE - 1;
		}
		
		if(DNARROW & elem)
		{
			t[RFILLOUT].ob_height = t[WDOWN].ob_y - t[RFILLOUT].ob_y - (D3DSIZE << 1) + 1;
		}
		else
		{
			if(SIZER & elem)
			{
				t[RFILLOUT].ob_height = t[WSIZER].ob_y - t[RFILLOUT].ob_y - (D3DSIZE << 1) + 1;
			}
			else
			{
				t[RFILLOUT].ob_height = t[WINDOW].ob_height - t[RFILLOUT].ob_y
					- bottomsize - D3DSIZE;
			}
		}
	}
	else
	{
		t[RFILLOUT].ob_flags |= HIDETREE;
	}
	
	if(LFARROW & elem)
	{
		t[WLEFT].ob_flags &= ~HIDETREE;
	}
	else
	{
		t[WLEFT].ob_flags |= HIDETREE;
	}
	
	if(RTARROW & elem)
	{
		t[WRIGHT].ob_flags &= ~HIDETREE;
		
		t[WRIGHT].ob_x = t[WINDOW].ob_width - rightsize
			- t[WRIGHT].ob_width - D3DSIZE + 1;
	}
	else
	{
		t[WRIGHT].ob_flags |= HIDETREE;
	}
	
	if(elem & HSLIDE)
	{
		t[WHSB].ob_flags &= ~HIDETREE;
		
		if(LFARROW & elem)
		{
			t[WHSB].ob_x = t[WLEFT].ob_width + (D3DSIZE * 3) - 1;
		}
		else
		{
			t[WHSB].ob_x = D3DSIZE;
		}
		
		if(RTARROW & elem)
		{
			t[WHSB].ob_width = t[WRIGHT].ob_x - t[WHSB].ob_x - (D3DSIZE << 1) + 1;
		}
		else
		{
			if((rightsize == 0) && (SIZER & elem))
			{
				t[WHSB].ob_width = t[WSIZER].ob_x - t[WHSB].ob_x - (D3DSIZE << 1) + 1;
			}
			else
			{
				t[WHSB].ob_width = t[WINDOW].ob_width - t[WHSB].ob_x
					- rightsize - (D3DSIZE << 1) + 1;
			}
		}
	}
	else
	{
		t[WHSB].ob_flags |= HIDETREE;
	}
	
	if(!(HSLIDE & elem) && (bottomsize > 0))
	{
		t[BFILLOUT].ob_flags &= ~HIDETREE;
		
		if(LFARROW & elem)
		{
			t[BFILLOUT].ob_x = t[WLEFT].ob_width;
		}
		else
		{
			t[BFILLOUT].ob_x = 0;
		}
		
		if(RTARROW & elem)
		{
			t[BFILLOUT].ob_width = t[WRIGHT].ob_x - t[BFILLOUT].ob_x;
		}
		else
		{
			if((rightsize == 0) && (SIZER & elem))
			{
				t[BFILLOUT].ob_width = t[WSIZER].ob_x - t[BFILLOUT].ob_x;
			}
			else
			{
				t[BFILLOUT].ob_width = t[WINDOW].ob_width - t[BFILLOUT].ob_x
					- rightsize;
			}
		}
	}
	else
	{
		t[BFILLOUT].ob_flags |= HIDETREE;
	}
	
	return(t);
}

static WORD	changeslider(WINSTRUCT *win,WORD redraw,WORD which,
				WORD position,WORD size) {	

	REDRAWSTRUCT m;

	if(which & VSLIDE) {
		if(position != -1) {
			if(position > 1000) {
				win->vslidepos = 1000;
			}
			else if(position < 1) {
				win->vslidepos = 1;
			}
			else {
				win->vslidepos = position;
			};
		};
		
		if(size != -1) {
			if(size > 1000) {
				win->vslidesize = 1000;
			}
			else if(size < 1) {
				win->vslidesize = 1;
			}
			else {
				win->vslidesize = size;
			};
		};
		
		win->tree[WVSLIDER].ob_height = ((LONG)win->vslidesize * (LONG)win->tree[WVSB].ob_height) / 1000L;

		win->tree[WVSLIDER].ob_y = ((LONG)win->vslidepos * (LONG)(win->tree[WVSB].ob_height - win->tree[WVSLIDER].ob_height)) / 1000L;
	};
	
	if(which & HSLIDE) {
		if(position != -1) {
			if(position > 1000) {
				win->hslidepos = 1000;
			}
			else if(position < 1) {
				win->hslidepos = 1;
			}
			else {
				win->hslidepos = position;
			};
		};
		
		if(size != -1) {
			if(size > 1000) {
				win->hslidesize = 1000;
			}
			else if(size < 1) {
				win->hslidesize = 1;
			}
			else {
				win->hslidesize = size;
			};
		};
		
		win->tree[WHSLIDER].ob_width = ((LONG)win->hslidesize * (LONG)win->tree[WHSB].ob_width) / 1000L;

		win->tree[WHSLIDER].ob_x = ((LONG)win->hslidepos * (LONG)(win->tree[WHSB].ob_width - win->tree[WHSLIDER].ob_width)) / 1000L;
	};

	if(redraw && (win->status & WIN_ISOPEN)) {							
		m.type = WM_REDRAW;
		m.length = 0;
		m.wid = win->id;
		
		if(which & VSLIDE) {
			Objc_area_needed(win->tree,WVSB,&m.area);

			if(globals.realmove) {
				m.area.x -= win->totsize.x;
				m.area.y -= win->totsize.y;
				m.sid = -1;
			}
			else {
				m.sid = 0;
			};

			Fwrite(drawque[1],sizeof(WMSG),&m.wid);
		};
		
		if(which & HSLIDE) {
			Objc_area_needed(win->tree,WHSB,&m.area);

			if(globals.realmove) {
				m.area.x -= win->totsize.x;
				m.area.y -= win->totsize.y;
				m.sid = -1;
			}
			else {
				m.sid = 0;
			};

			Fwrite(drawque[1],sizeof(WMSG),&m.wid);
		};
	};

	return 1;
}

/*setwinsize sets the size and position of window win to size*/

static void	setwinsize(WINSTRUCT *win,RECT *size) {
	WORD	dx,dy,dw,dh;

	win->lastsize = win->totsize;
	
	win->totsize = *size;
	
	dx = size->x - win->lastsize.x;
	dy = size->y - win->lastsize.y;
	dw = size->width - win->lastsize.width;
	dh = size->height - win->lastsize.height;
	
	win->worksize.x += dx;
	win->worksize.y += dy;
	win->worksize.width += dw;
	win->worksize.height += dh;

	if(win->tree) {
		win->tree[0].ob_x = win->totsize.x;
		win->tree[0].ob_y = win->totsize.y;
		win->tree[0].ob_width = win->totsize.width;
		win->tree[0].ob_height = win->totsize.height;
	
		win->tree[WMOVER].ob_width += dw;
		
		win->tree[WFULLER].ob_x += dw;
	
		win->tree[WDOWN].ob_x += dw;
		win->tree[WDOWN].ob_y += dh;	
	
		win->tree[WSIZER].ob_x += dw;
		win->tree[WSIZER].ob_y += dh;	
	
		win->tree[WRIGHT].ob_x += dw;
		win->tree[WRIGHT].ob_y += dh;	
	
		win->tree[WLEFT].ob_y += dh;	
	
		win->tree[WVSB].ob_x += dw;
		win->tree[WVSB].ob_height += dh;	
	
		win->tree[WHSB].ob_y += dh;
		win->tree[WHSB].ob_width += dw;	
	
		win->tree[WINFO].ob_width += dw;
		
		win->tree[WUP].ob_x += dw;
		
		win->tree[TFILLOUT].ob_width += dw;
	
		win->tree[RFILLOUT].ob_height += dh;
		win->tree[RFILLOUT].ob_x += dw;
	
		win->tree[BFILLOUT].ob_width += dw;
		win->tree[BFILLOUT].ob_y += dy;
	
		win->tree[SFILLOUT].ob_x += dw;
		win->tree[SFILLOUT].ob_y += dh;
		
		changeslider(win,0,HSLIDE | VSLIDE,-1,-1);
	};
}

/*winalloc creates/fetches a free window structure/id*/

static WINSTRUCT	*winalloc(void) {
	WINLIST	*wl;
	
	if(win_free) {
		wl = win_free;
		win_free = win_free->next;
	}
	else {
		wl = (WINLIST *)Mxalloc(sizeof(WINLIST),GLOBALMEM);
		wl->win = (WINSTRUCT *)Mxalloc(sizeof(WINSTRUCT),GLOBALMEM);
		wl->win->id = win_next;
		win_next++;
	};
	
	wl->next = win_list[wl->win->id % WIN_NR];
	win_list[wl->win->id % WIN_NR] = wl;
	
	return wl->win;
}

/* delwinelem deletes an object created with crwinelem*/

static void	delwinelem(OBJECT *o)
{
	if(o)
		Mfree(o);
}

static WORD	top_window(WORD winid) { 
	REDRAWSTRUCT	m;
	
	RLIST	*rl = 0L;
	RLIST	*rl2 = 0L;
	
	WINLIST	**wl = &globals.win_vis;
	WINLIST	*wl2;
	WINLIST	*ourwl;
	
	WORD	dx,dy;
	
	while(*wl) {
		if((*wl)->win->id == winid)
			break;
			
		wl = &(*wl)->next;
	};
	
	if(!*wl)
		return 0;
	
	wl2 = *wl;
	
	*wl = (*wl)->next;

	if(globals.win_vis) {
		if(globals.win_vis->win->status & WIN_DIAL) {
			wl2->next = globals.win_vis->next;
			globals.win_vis->next = wl2;
		}
		else {
			COMMSG m;
			
			wl2->next = globals.win_vis;
			globals.win_vis = wl2;
			
			m.type = MH_TOP;
			m.sid = 0;
			m.length = 0;
			m.msg0 = wl2->win->owner;

			Appl_do_write(0, 16, &m);
		};
	}
	else
	{	
		wl2->next = 0L;
		globals.win_vis = wl2;
	};

	m.type = WM_REDRAW;
	
	if(globals.realmove) {
		m.sid = -1;
	}
	else {
		m.sid = 0;
	};
	
	m.length = 0;
	m.wid = wl2->win->id;
	ourwl = wl2;
	dx = wl2->win->totsize.x;
	dy = wl2->win->totsize.y;	
		
	wl2 = wl2->next;

	while(wl2) {
		RLIST	*rd = 0L;
		
		Rlist_rectinter(&rl,&ourwl->win->totsize,&wl2->win->rlist);
		
		Rlist_insert(&rd,&wl2->win->rlist);
			
		wl2->win->rlist = rd;
		wl2->win->rpos = wl2->win->rlist;
		
		wl2 = wl2->next;
	};
	

	Rlist_insert(&rl2,&rl);
	
	rl = rl2;

	
	while(rl) {
		m.area.x = rl->r.x;
		m.area.y = rl->r.y;
		
		if(globals.realmove) {
			m.area.x -= dx;
			m.area.y -= dy;
		};
		
		m.area.width = rl->r.width;
		m.area.height = rl->r.height;
	
		Fwrite(drawque[1],sizeof(WMSG),&m.wid);
		Appl_do_write(ourwl->win->owner,16,&m);

		rl = rl->next;
	};

	Rlist_insert(&ourwl->win->rlist,&rl2);

	ourwl->win->rpos = ourwl->win->rlist;

	return 1;
}

static WORD	changewinsize(WINSTRUCT *win,RECT *size,WORD vid) {	
	if(win->status & WIN_ISOPEN) {
		WORD	dx = size->x - win->totsize.x;
		WORD	dy = size->y - win->totsize.y;
		WORD	dw = size->width - win->totsize.width;
		WORD	dh = size->height - win->totsize.height;


		if(dx || dy) { /* pos (and maybe size) is to be changed */
			REDRAWSTRUCT	m;
			
			WINLIST	*wl = globals.win_vis;
			
			while(wl)
			{
				if(wl->win == win)
				{
					wl = wl->next;
					break;
				};
					
				wl = wl->next;
			};
			
			if(wl) {
				RLIST	*rlwalk;
				RLIST	*rlournew = NULL;
				RLIST	*rlourold = win->rlist;
				RLIST	*old_rlist = Rlist_duplicate(win->rlist);
				RLIST	*rlmove = NULL;
				RLIST *rlmove1 = NULL;
			
				win->rlist = 0L;
			
				/*grab the rectangles we need from our old list*/
				
				Rlist_rectinter(&rlournew,size,&rlourold);
			
				while(wl) {
					RLIST	*rltheirnew = NULL;
					
					/*get the rectangles we need*/
					
					Rlist_rectinter(&rlournew,size,&wl->win->rlist);
					
					/*give away the rectangles we don't need*/
					
					Rlist_rectinter(&rltheirnew,&wl->win->totsize,&rlourold);
				
					/*update the new rectangles*/
				
					m.type = WM_REDRAW;
					
					if(globals.realmove) {
						m.sid = -1;
					}
					else {
						m.sid = 0;
					};
					
					m.length = 0;
					m.wid = wl->win->id;
				
					rlwalk = rltheirnew;
					
					while(rlwalk) {
						m.area.x = rlwalk->r.x;
						m.area.y = rlwalk->r.y;
						
						if(globals.realmove) {
							m.area.x -= wl->win->totsize.x;
							m.area.y -= wl->win->totsize.y;
						};
						
						m.area.width = rlwalk->r.width;
						m.area.height = rlwalk->r.height;
					
						Fwrite(drawque[1],sizeof(WMSG),&m.wid);
						Appl_do_write(wl->win->owner,16,&m);
				
						rlwalk = rlwalk->next;
					};				
				
					Rlist_insert(&wl->win->rlist,&rltheirnew);

					wl->win->rpos = wl->win->rlist;

					wl = wl->next;
				};
				
				Rlist_insert(&win->rlist,&rlournew);
			
				/*figure out which rectangles that are moveable*/

				if(dw || dh) {
					RECT new_work;
					
					new_work.x = win->worksize.x + dx;
					new_work.y = win->worksize.y + dy;
					new_work.width = win->worksize.width + dw;
					new_work.height = win->worksize.height + dh;
					
					Rlist_rectinter(&rlmove1,&new_work,&win->rlist);
				}
				else {
					rlmove1 = win->rlist;
					win->rlist = NULL;
				};
											
				rlwalk = old_rlist;
				
				while(rlwalk) {
					rlwalk->r.x += dx;
					rlwalk->r.y += dy;
					
					Rlist_rectinter(&rlmove,&rlwalk->r,&rlmove1);
					
					rlwalk = rlwalk->next;
				};
				
				/*move the rectangles that are moveable*/

				Rlist_sort(&rlmove,dx,dy);
				
				rlwalk = rlmove;

				Vdi_v_hide_c(vid);

				while(rlwalk) {
					MFDB	mfdbd,mfdbs;
					WORD	koordl[8];
					
					mfdbd.fd_addr = 0L;
					mfdbs.fd_addr = 0L;
					
					koordl[4] = rlwalk->r.x;
					koordl[5] = rlwalk->r.y + rlwalk->r.height - 1;
					koordl[6] = rlwalk->r.x + rlwalk->r.width - 1;
					koordl[7] = rlwalk->r.y;
					koordl[0] = koordl[4] - dx;
					koordl[1] = koordl[5] - dy;
					koordl[2] = koordl[6] - dx;
					koordl[3] = koordl[7] - dy;
					
					Vdi_vro_cpyfm(vid,S_ONLY,koordl,&mfdbs,&mfdbd);
					
					rlwalk = rlwalk->next;
				};

				/*update rectangles that are not moveable*/
							
				m.type = WM_REDRAW;
				
				if(globals.realmove) {
					m.sid = -1;
				}
				else {
					m.sid = 0;
				};
				
				m.length = 0;
				m.wid = win->id;

				Rlist_insert(&win->rlist,&rlmove1);
			
				rlwalk = win->rlist;
				
				while(rlwalk) {
					m.area.x = rlwalk->r.x;
					m.area.y = rlwalk->r.y;
					
					if(globals.realmove) {
						m.area.x -= size->x;
						m.area.y -= size->y;
					};
					
					m.area.width = rlwalk->r.width;
					m.area.height = rlwalk->r.height;
				
					Fwrite(drawque[1],sizeof(WMSG),&m.wid);
					Appl_do_write(win->owner,16,&m);

					rlwalk = rlwalk->next;
				};
								
				Vdi_v_show_c(vid,1);

				Rlist_insert(&win->rlist,&rlmove);
				
				Rlist_erase(&old_rlist);

				win->rpos = win->rlist;
			};			
		}
		else if(dw || dh)	/*only size changed*/
		{
			RECT	rt;
			RECT	dn;

			REDRAWSTRUCT	m;

			RLIST	*rl = 0L;
			RLIST	*rl2;
			RLIST	*rltop = 0L;
			
			WINLIST	*wl = globals.win_vis;
						

			rt.x = size->x + size->width;

			if(dw < 0) {
				rt.width = -dw;
			}
			else {
				rt.x -= dw;
				rt.width = dw;
			};
			
			rt.y = size->y;
			rt.height = size->height;
				
			dn.y = size->y + size->height;

			if(dh < 0) {
				dn.height = -dh;
				dn.width = win->totsize.width;
			}
			else {
				dn.y -= dh;
				dn.height = dh;
				dn.width = size->width;
			};

			dn.x = size->x;
		
			if(dw < 0) {
				Rlist_rectinter(&rl,&rt,&win->rlist);
			};
		
			if(dh < 0) {
				Rlist_rectinter(&rl,&dn,&win->rlist);
			};
		
			while(wl) {
				if(wl->win == win) {
					wl = wl->next;
					break;
				};
					
				wl = wl->next;
			};
		
			while(wl) {
				RLIST	*rd = 0;

				if(dw < 0) {
					Rlist_rectinter(&rd,&wl->win->totsize,&rl);
				}
				else if(dw > 0) {
					Rlist_rectinter(&rltop,&rt,&wl->win->rlist);
				};
				
				if(dh < 0) {
					Rlist_rectinter(&rd,&wl->win->totsize,&rl);
				}
				else if(dh > 0) {
					Rlist_rectinter(&rltop,&dn,&wl->win->rlist);
				};
			
				m.type = WM_REDRAW;
				
				if(globals.realmove) {
					m.sid = -1;
				}
				else {
					m.sid = 0;
				};
				
				m.length = 0;
				m.wid = wl->win->id;
			
				rl2 = rd;
				
				while(rl2) {
					m.area.x = rl2->r.x;
					m.area.y = rl2->r.y;
					
					if(globals.realmove) {
						m.area.x -= wl->win->totsize.x;
						m.area.y -= wl->win->totsize.y;
					};
					
					m.area.width = rl2->r.width;
					m.area.height = rl2->r.height;
				
					Fwrite(drawque[1],sizeof(WMSG),&m.wid);
					Appl_do_write(wl->win->owner,16,&m);

					rl2 = rl2->next;
				};
				
				Rlist_insert(&wl->win->rlist,&rd);
				
				wl->win->rpos = wl->win->rlist;
				
				wl = wl->next;
			};

			rl2 = 0;

			Rlist_rectinter(&rl2,&win->worksize,&win->rlist);

			Rlist_insert(&rltop,&win->rlist);
			
			Rlist_insert(&win->rlist,&rl2);

			m.wid = win->id;
			
			if(globals.realmove) {
				m.area.x = 0;
				m.area.y = 0;
			}
			else {
				m.area.x = size->x;
				m.area.y = size->y;
			};
			
			m.area.width = size->width;
			m.area.height = size->height;
		
			Fwrite(drawque[1],sizeof(WMSG),&m.wid);
			
			rl2 = rltop;
			
			while(rl2) {
				m.area.x = rl2->r.x;
				m.area.y = rl2->r.y;
				
				if(globals.realmove) {
					m.area.x -= size->x;
					m.area.y -= size->y;
				};
				
				m.area.width = rl2->r.width;
				m.area.height = rl2->r.height;
				
				Appl_do_write(win->owner,16,&m);

				rl2 = rl2->next;
			};
			
			Rlist_insert(&win->rlist,&rltop);

			win->rpos = win->rlist;
		};
	};
	
	setwinsize(win,size);
	
	return 1;
}

/*calcworksize calculates the worksize or the total size of
a window. If dir == WC_WORK the worksize will be calculated and 
otherwise the total size will be calculated.*/

static void	calcworksize(WORD elem,RECT *orig,RECT *new,WORD dir) {
	WORD	bottomsize = 1;
	WORD	headsize = 1;
	WORD	leftsize = 1;
	WORD	rightsize = 1;
	WORD	topsize;
	
	if((HSLIDE | LFARROW | RTARROW) & elem) {
		bottomsize = windowtad[WLEFT].ob_height + (D3DSIZE << 1);
	};
	
	if((CLOSER | MOVER | FULLER) & elem) {
		topsize = windowtad[WMOVER].ob_height + (D3DSIZE << 1);
	}
	else {
		topsize = 0;
	};
	
	if(INFO & elem) {
		headsize = topsize + windowtad[WINFO].ob_height - 1;
	}
	else {
		if(topsize)
			headsize = topsize;
		else
			headsize = 1;
	};
	
	if((LFARROW | HSLIDE | RTARROW) & elem) {
		bottomsize = windowtad[WLEFT].ob_height + (D3DSIZE << 1);
	};
	
	if(((bottomsize < windowtad[WLEFT].ob_height) && (SIZER & elem))
		|| ((VSLIDE | UPARROW | DNARROW) & elem))
	{
		rightsize = windowtad[WSIZER].ob_width + (D3DSIZE << 1);
	};

	if(dir == WC_WORK) {
		new->x = orig->x + leftsize;
		new->y = orig->y + headsize;
		new->width = orig->width - leftsize - rightsize;
		new->height = orig->height - headsize - bottomsize;
	}
	else {
		new->x = orig->x - leftsize;
		new->y = orig->y - headsize;
		new->width = orig->width + leftsize + rightsize;
		new->height = orig->height + headsize + bottomsize;
	};
}

/****************************************************************************
 * Public functions                                                         *
 ****************************************************************************/

/*init_wind initializes the window system*/

void	init_wind(void) {
  WORD	vid = globals.vid;
  WORD	i;
  WORD	work_in[] = {1,7,1,1,1,1,1,1,1,1,2};
  WORD	work_out[57];
  
  WINLIST	*wl;
  
  WINSTRUCT	*ws;
	
  
  /* open up a vdi workstation to use in the process */
  
  Vdi_v_opnvwk(work_in,&vid,work_out);
	
  recalc_tree(windowtad,globals.clwidth,globals.clheight);

  for(i = 0; i < WIN_NR; i++) {
    win_list[i] = 0L;
  }
  
  if(globals.num_pens < 16) {
  	for(i = 0; i <= W_SMALLER; i++) {
  		untop_colour[i].pattern = 0;
  		untop_colour[i].fillc = WHITE;
  		top_colour[i].pattern = 0;
  		top_colour[i].fillc = WHITE;
  	}
  }
	
  ws = winalloc();
  ws->status = WIN_ISOPEN | WIN_DESKTOP | WIN_UNTOPPABLE;
  ws->owner = 0; 
  
  ws->maxsize = globals.screen;
  
  ws->tree = NULL;
  deskbgtad[0].ob_x = globals.screen.x;
  deskbgtad[0].ob_y = globals.screen.y + globals.clheight + 3;
  deskbgtad[0].ob_width = globals.screen.width;
  deskbgtad[0].ob_height = globals.screen.height - globals.clheight - 3;
  
  ws->totsize = globals.screen;
  
  ws->worksize.x = globals.screen.x;
  ws->worksize.y = globals.screen.y + globals.clheight + 3;
  ws->worksize.width = globals.screen.width;
  ws->worksize.height = globals.screen.height - globals.clheight - 3;
  
  wl = (WINLIST *)Mxalloc(sizeof(WINLIST),GLOBALMEM);
  
  wl->win = findwin(0);
  
  wl->next = globals.win_vis;
  globals.win_vis = wl;

  wl->win->rlist = (RLIST *)Mxalloc(sizeof(RLIST),GLOBALMEM);
  wl->win->rlist->r.x = globals.screen.x;
  wl->win->rlist->r.y = globals.screen.y;
  wl->win->rlist->r.width = globals.screen.width;
  wl->win->rlist->r.height = globals.screen.height;
  wl->win->rlist->next = NULL;
  
  drawwinelem(vid,wl->win,&wl->win->rlist->r,0);

	Fpipe(drawque);

	drawid = (WORD)newfork(drawproc,0,"DrawWind");
  
  Fclose(drawque[0]);
  
  Vdi_v_clsvwk(vid);
  
  /* Create screen drawing semaphore */
  
  Psemaphore(0,DRAW_SEM,-1);
  Psemaphore(3,DRAW_SEM,-1);
}

void	exit_wind(void) {
	Pkill(drawid,SIGKILL);

	Fclose(drawque[1]);

  /* Destroy screen drawing semaphore */
  
  Psemaphore(2,DRAW_SEM,-1);
  Psemaphore(1,DRAW_SEM,-1);
}


/****************************************************************************
 * Wind_update_deskbg                                                       *
 *  Update all of the desk background.                                      *
 ****************************************************************************/
void                     /*                                                 */
Wind_update_deskbg(void) /*                                                 */
/****************************************************************************/
{
	WMSG m;
	
	m.wid = 0;
	m.r = globals.screen;
	
	Fwrite(drawque[1],sizeof(WMSG),&m);
}

/*findwin finds the window structure of the window with
identification number id*/

WINSTRUCT	*findwin(WORD id) {
	WINLIST	*wl;

	wl = win_list[id % WIN_NR];
	
	while(wl) {
		if(wl->win->id == id)
		{
			return wl->win;
		}

		wl = wl->next;
	};

	return NULL;
}

/*delete_window frees a window structure allocated by winalloc*/
void	delete_window(WORD id) {
	WINLIST	**wl;
		
	wl = &win_list[id % WIN_NR];
	
	while(*wl)
	{		
		if((*wl)->win->id == id)
			break;
	
		wl = &(*wl)->next;
	};
	
	if(!(*wl))
	{
		/*no such window found! return*/
		return;
	};
	
	if((*wl)->win->status & WIN_ISOPEN)
	{
		close_window(id);
	};
	
	delwinelem((*wl)->win->tree);
	
	if(id == (win_next - 1))
	{
		WORD	found;
		WINLIST	*wt = *wl;
		
		*wl = (*wl)->next;
		
		Mfree(wt->win);
		Mfree(wt);
		win_next--;
		
		do
		{
			WINLIST 	**wlwalk = &win_list[(win_next - 1) % WIN_NR];
			
			found = 0;
			
			while(*wlwalk)
			{
				if((*wlwalk)->win->id == (win_next - 1))
				{
					found = 1;
					break;
				};
				
				wlwalk = &(*wlwalk)->next;
			};
			
			if(!found)
			{
				wlwalk = &win_free;
				
				while(*wlwalk)
				{
					if((*wlwalk)->win->id == (win_next - 1))
					{
						WINLIST	*wltemp = *wlwalk;
						
						*wlwalk = (*wlwalk)->next;
						
						Mfree(wltemp->win);
						Mfree(wltemp);
						
						win_next--;
						
						break;
					};
					
					wlwalk = &(*wlwalk)->next;
				};
			};
			
		}while(!found && win_next);

	}
	else
	{
		WINLIST	*wt = *wl;
		
		*wl = (*wl)->next;
		
		wt->next = win_free;
		win_free = wt;
	};
}



/* drawwinelem draws the elements of the window win that intersects
with the rectangle r*/

void	drawwinelem(WORD vid,WINSTRUCT *win,RECT *r,WORD start) {
	RLIST	*rl = win->rlist;

	if(win->id == 0) {
		OBJECT *deskbg;
	
		Rdwrs_operation(ASTARTREAD);
		
		deskbg = Appl_get_deskbg(DESK_OWNER);
		
		if(deskbg) {
			while(rl) {		
				RECT	r2;
			
				if(Misc_intersect(&rl->r,r,&r2)) {
					Objc_do_draw(vid,deskbg,start,9,&r2);
				};
					
				rl = rl->next;
			};
		};
		
		Rdwrs_operation(AENDREAD);		
	}
	else if(win->tree) {	
		while(rl) {		
			RECT	r2;
		
			if(Misc_intersect(&rl->r,r,&r2)) {
				Objc_do_draw(vid,win->tree,start,3,&r2);
			};
			rl = rl->next;
		};
	};
}


WORD	create_window(WORD owner,WORD elems,RECT *maxsize,WORD status) {
	WINSTRUCT	*ws;
	
	ws = winalloc();
	ws->status = status;
	ws->owner = owner; 
	
	ws->maxsize = *maxsize;

	ws->rlist = NULL;
	ws->rpos = NULL;

	ws->vslidepos = 1;
	ws->vslidesize = 1000;
	ws->hslidepos = 1;
	ws->hslidesize = 1000;
	
	if((ws->status & WIN_DIAL) || (ws->status & WIN_MENU))
	{
		ws->tree = 0L;
		ws->totsize = *maxsize;
		ws->worksize = ws->totsize;
	}
	else
	{
		WORD i;
		
		ws->tree = crwinelem(elems);
	
		ws->totsize.x = ws->tree[0].ob_x;
		ws->totsize.y = ws->tree[0].ob_y;
		ws->totsize.width = ws->tree[0].ob_width;
		ws->totsize.height = ws->tree[0].ob_height;
	
		calcworksize(elems,&ws->totsize,&ws->worksize,WC_WORK);
		
		for(i = 0; i <= W_SMALLER; i++) {
			ws->top_colour[i] = top_colour[i];
			ws->untop_colour[i] = untop_colour[i];
		}
		
		ws->own_colour = 0;
	};

	return ws->id;
}

/*wind_create 0x0064*/

void	Wind_create(AES_PB *apb) {	
	Rdwrs_operation(WSTARTWRITE);

	apb->int_out[0] = create_window(apb->global[2]
		,apb->int_in[0],(RECT *)&apb->int_in[1],0);

	Rdwrs_operation(WENDWRITE);
}


WORD open_window(WORD id,RECT *size) {
	WINLIST	*wl,*wp;
	WINSTRUCT *ws;
	RLIST	*rl = 0L;
	REDRAWSTRUCT	m;
	WORD	owner;
	WORD  i;

	ws = findwin(id);

	if(ws) {
		if(!(ws->status & WIN_ISOPEN)) {
			wl = (WINLIST *)Mxalloc(sizeof(WINLIST),GLOBALMEM);
		
			wl->win = ws;
			
			setwinsize(wl->win,size);
		
			if(globals.win_vis) {
				if(globals.win_vis->win->status & WIN_DIAL) {
					wl->next = globals.win_vis->next;
					globals.win_vis->next = wl;
				}
				else
				{					
					wl->next = globals.win_vis;
					globals.win_vis = wl;

					if(!(wl->win->status & WIN_MENU)) {
						COMMSG m;
	
						m.type = MH_TOP;
						m.sid = 0;
						m.length = 0;
						m.msg0 = wl->win->owner;

						Appl_do_write(0, 16, &m);
					};
				};
			}
			else {	
				wl->next = 0L;
				globals.win_vis = wl;
			};
			
			wp = wl->next;
		
			while(wp != 0L) {
				RLIST	*rd = 0L;
				
				Rlist_rectinter(&rl,&wl->win->totsize,&wp->win->rlist);
				
				Rlist_insert(&rd,&wp->win->rlist);
					
				wp->win->rlist = rd;
				wp = wp->next;
			};
		
			wl->win->rlist = 0L;
			Rlist_insert(&wl->win->rlist,&rl);
		
			wl->win->status |= WIN_ISOPEN;
			
			for(i = 0; i <= W_SMALLER; i++) {
				set_widget_colour(ws,i,&ws->untop_colour[i],&ws->top_colour[i]);
			};
			
			if(!(wl->win->status & WIN_DIAL)){
				m.type = WM_REDRAW;
				m.length = 0;
				
				if(globals.realmove) {
					m.sid = -1;
					m.area.x = 0; /*x and y are relative to the position of the window*/
					m.area.y = 0;
				}
				else {
					m.sid = 0;
					m.area.x = wl->win->totsize.x;
					m.area.y = wl->win->totsize.y;
				};
				
				m.area.width = wl->win->totsize.width;
				m.area.height = wl->win->totsize.height;
				
				m.wid = wl->win->id;
			
				owner = wl->win->owner;
				
				Fwrite(drawque[1],sizeof(WMSG),&m.wid);
			
				Appl_do_write(owner,16,&m);
			};
		}
		else {
			return 0;
		};
	}
	else {
		return 0;
	};
	
	return 1;
}

/*wind_open 0x0065*/

void Wind_open(AES_PB *apb) {
	Rdwrs_operation(WSTARTWRITE);

	apb->int_out[0] = open_window(apb->int_in[0],(RECT *)&apb->int_in[1]);

	Rdwrs_operation(WENDWRITE);
}

/*wind_close 0x0066*/

WORD	close_window(WORD wid) {
	WINLIST	**wl = &globals.win_vis;
	
	while(*wl) {
		if((*wl)->win->id == wid)
			break;
			
		wl = &(*wl)->next;
	};
	
	if(*wl) {
		WINLIST	*wp = (*wl)->next;
		
		while(wp) {
			
			RLIST	*rl = NULL;
			RLIST	*tl;
			
			Rlist_rectinter(&rl,&wp->win->totsize,&(*wl)->win->rlist);
			
			tl = rl;
			
			while(tl) {
				REDRAWSTRUCT	m;
				
				m.type = WM_REDRAW;
				m.length = 0;
				m.wid = wp->win->id;
				
				if(globals.realmove) {
					m.sid = -1;
					m.area.x = tl->r.x - wp->win->totsize.x;
					m.area.y = tl->r.y - wp->win->totsize.y;
				}
				else {
					m.sid = 0;
					m.area.x = tl->r.x;
					m.area.y = tl->r.y;
				};
				
				m.area.width = tl->r.width;
				m.area.height = tl->r.height;	

				Fwrite(drawque[1],sizeof(WMSG),&m.wid);

				Appl_do_write(wp->win->owner,16,&m);

				tl = tl->next;
			}
						
			Rlist_insert(&wp->win->rlist,&rl);

			wp->win->rpos = wp->win->rlist;

			wp = wp->next;
		}		
		
		wp = *wl;
		
		*wl = (*wl)->next;

		wp->win->status &= ~WIN_ISOPEN;
		Mfree(wp);
		
		return 1;
	}
	else
	{
		return 0;
	}
}

void	Wind_close(AES_PB *apb) {
	Rdwrs_operation(WSTARTWRITE);

	apb->int_out[0] = close_window(apb->int_in[0]);

	Rdwrs_operation(WENDWRITE);
}


/*wind_delete 0x0067*/
void	Wind_delete(AES_PB *apb) {
	Rdwrs_operation(WSTARTWRITE);
	delete_window(apb->int_in[0]);
	Rdwrs_operation(WENDWRITE);
}

/*wind_get 0x0068*/
void	Wind_get(AES_PB *apb) {
	WINSTRUCT	*win;
	
	Rdwrs_operation(WSTARTREAD);
	
	win = findwin(apb->int_in[0]);

	if(win) {
		switch(apb->int_in[1]) {	
			case WF_WORKXYWH	:	/*0x0004*/
				apb->int_out[0] = 1;
				apb->int_out[1] = win->worksize.x;
				apb->int_out[2] = win->worksize.y;
				apb->int_out[3] = win->worksize.width;
				apb->int_out[4] = win->worksize.height;
				break;
			
			case	WF_CURRXYWH	:	/*0x0005*/
				apb->int_out[0] = 1;
				apb->int_out[1] = win->totsize.x;
				apb->int_out[2] = win->totsize.y;
				apb->int_out[3] = win->totsize.width;
				apb->int_out[4] = win->totsize.height;
				break;

			case	WF_PREVXYWH	:	/*0x0006*/
				apb->int_out[0] = 1;
				apb->int_out[1] = win->lastsize.x;
				apb->int_out[2] = win->lastsize.y;
				apb->int_out[3] = win->lastsize.width;
				apb->int_out[4] = win->lastsize.height;
				break;
			
			case	WF_FULLXYWH	:	/*0x0007*/
				apb->int_out[0] = 1;
				apb->int_out[1] = win->maxsize.x;
				apb->int_out[2] = win->maxsize.y;
				apb->int_out[3] = win->maxsize.width;
				apb->int_out[4] = win->maxsize.height;
				break;
				
			case WF_HSLIDE: /*0x08*/
				apb->int_out[0] = 1;
				apb->int_out[1] = win->hslidepos;
				break;

			case WF_VSLIDE: /*0x09*/
				apb->int_out[0] = 1;
				apb->int_out[1] = win->vslidepos;
				break;

			case WF_TOP: /*0x000a*/
				apb->int_out[1] = globals.win_vis->win->id;
				apb->int_out[2] = globals.win_vis->win->owner;
				
				if(globals.win_vis->next) {
					apb->int_out[3] = globals.win_vis->next->win->id;
				}
				else {
					apb->int_out[3] = -1;
				};
				
				apb->int_out[0] = 1;
				break;

			case	WF_FIRSTXYWH:	/*0x000b*/
				win->rpos = win->rlist;
			case	WF_NEXTXYWH:	/*0x000c*/
				apb->int_out[0] = 1;
				
				while(win->rpos) {
					if(Misc_intersect(&win->rpos->r,&win->worksize
						,(RECT *)&apb->int_out[1]))
						break;
					
					win->rpos = win->rpos->next;
				};					
				
				if(!win->rpos) {
					apb->int_out[1] = 0;
					apb->int_out[2] = 0;
					apb->int_out[3] = 0;
					apb->int_out[4] = 0;
				}
				else	
					win->rpos = win->rpos->next;
				
				break;

			case WF_HSLSIZE: /*0x0f*/
				apb->int_out[0] = 1;
				apb->int_out[1] = win->hslidesize;
				break;				

			case WF_VSLSIZE: /*0x10*/
				apb->int_out[0] = 1;
				apb->int_out[1] = win->vslidesize;
				break;				

			case WF_SCREEN: /*0x11*/
				apb->int_out[0] = 1;
				apb->int_out[1] = 0;
				apb->int_out[2] = 0;
				apb->int_out[3] = 0;
				apb->int_out[4] = 0;
				break;
				
			case WF_OWNER: /*0x14*/
				{
					WINSTRUCT *ws = findwin(apb->int_in[0]);
				
					if(ws) {
						apb->int_out[0] = 1;
						apb->int_out[1] = ws->owner;
						if(ws->status & WIN_ISOPEN) {
							WINLIST *wl = globals.win_vis;
							
							apb->int_out[2] = 1;

							if(wl->win == ws) {
								apb->int_out[3] = -1;
							}
							else {
								while(wl->next) {
									if(wl->next->win == ws) {
										apb->int_out[3] = wl->win->id;
										break;
									};
									
									wl = wl->next;
								};
								
								wl = wl->next;
							};
							
							if(wl) {
								if(wl = wl->next) {
									apb->int_out[4] = wl->win->id;
								}
								else {
									apb->int_out[4] = -1;
								};
							}
							else {
								apb->int_out[4] = -1;
							};							
						}
						else {
							apb->int_out[2] = 0;
							apb->int_out[3] = -1;
							apb->int_out[4] = -1;
						};
					}
					else {
						apb->int_out[0] = 0;
					};
				};
				break;

			case WF_WINX:
			case WF_WINXCFG:
				apb->int_out[0] = 0;
				break;

			default:
				DB_printf("%s: Line %d: Wind_get:\r\n"
								"Unknown mode %d  (0x%x) wind_get(%d,%d,...)",
								__FILE__,__LINE__,apb->int_in[1],apb->int_in[1],
								apb->int_in[0],apb->int_in[1]);
				apb->int_out[0] = 0;
		};
	}
	else
		apb->int_out[0] = 0;

	Rdwrs_quick(WENDREAD);	
}


/*wind_set 0x0069*/
void	Wind_set(AES_PB *apb) {
	WINSTRUCT	*win;
	
	Rdwrs_operation(WSTARTWRITE);
	
	win = findwin(apb->int_in[0]);
	

	if(win) {
		WMSG m;

		switch(apb->int_in[1]) {	
			case	WF_NAME	:	/*0x0002*/
				Mfree(win->tree[WMOVER].ob_spec.tedinfo->te_ptext);
				win->tree[WMOVER].ob_spec.tedinfo->te_ptext =
					Mxalloc(strlen(((char **)&apb->int_in[2])[0]) + 1,GLOBALMEM);
				strcpy(win->tree[WMOVER].ob_spec.tedinfo->te_ptext,((char **)&apb->int_in[2])[0]);
				m.wid = win->id;
				
				Objc_area_needed(win->tree,WMOVER,&m.r);

				if(globals.realmove) {
					m.r.x -= win->totsize.x;
					m.r.y -= win->totsize.y;
				};

				Fwrite(drawque[1],sizeof(WMSG),&m);

				apb->int_out[0] = 1;
				break;

			case	WF_INFO	:	/*0x0003*/
				Mfree(win->tree[WINFO].ob_spec.tedinfo->te_ptext);
				win->tree[WINFO].ob_spec.tedinfo->te_ptext =
					Mxalloc(strlen(((char **)&apb->int_in[2])[0]) + 1,GLOBALMEM);
				strcpy(win->tree[WINFO].ob_spec.tedinfo->te_ptext,((char **)&apb->int_in[2])[0]);

				m.wid = win->id;
				
				Objc_area_needed(win->tree,WINFO,&m.r);

				if(globals.realmove) {
					m.r.x -= win->totsize.x;
					m.r.y -= win->totsize.y;
				};

				Fwrite(drawque[1],sizeof(WMSG),&m);
				apb->int_out[0] = 1;
				break;
				
			case	WF_CURRXYWH	:	/*0x0005*/
				{
					WORD	vid = Appl_get_vid(apb->global[2]);
					
					if(vid > 0) {
						apb->int_out[0] = 
							changewinsize(win,(RECT *)&apb->int_in[2],vid);
					}
					else {
						apb->int_out[0] = 0;
					};
				};
				break;

			case	WF_HSLIDE: /*0x08*/
				apb->int_out[0] = 
					changeslider(win,1,HSLIDE,apb->int_in[2],-1);
				break;

			case	WF_VSLIDE: /*0x09*/
				apb->int_out[0] = 
					changeslider(win,1,VSLIDE,apb->int_in[2],-1);
				break;

			case	WF_TOP	:	/*0x000a*/
				apb->int_out[0] = top_window(apb->int_in[0]);
				break;

			case WF_NEWDESK: /*0x000e*/
				{
					WINSTRUCT *win = findwin(0);
					
					if(win) {
						*((RECT *)&(*(OBJECT **)&apb->int_in[2])[0].ob_x) =
							win->worksize;
					};
					
					if(Appl_set_deskbg(apb->global[2],((OBJECT **)&apb->int_in[2])[0]) == 0) {					
						apb->int_out[0] = 1;
					}
					else {
						apb->int_out[0] = 0;
					};
										
					if(apb->int_out[0]) {
						REDRAWSTRUCT	m;

						m.type = WM_REDRAW;
						m.sid = 0;
						m.length = 0;
						m.wid = 0;
				
						m.area = globals.screen;
					
						Fwrite(drawque[1],sizeof(WMSG),&m.wid);
					};
				};
				break;

			case	WF_HSLSIZE: /*0x0f*/
				apb->int_out[0] = 
					changeslider(win,1,HSLIDE,-1,apb->int_in[2]);
				break;

			case	WF_VSLSIZE: /*0x10*/
				apb->int_out[0] = 
					changeslider(win,1,VSLIDE,-1,apb->int_in[2]);
				break;

			case WF_COLOR:  /*0x12*/
				DB_printf("Wind_set WF_COLOR not implemented");
				apb->int_out[0] = 0;
				break;

			case WF_DCOLOR: /*0x13*/
				top_colour[apb->int_in[2]] = *(OBJC_COLORWORD *)&apb->int_in[3];
				untop_colour[apb->int_in[2]] = *(OBJC_COLORWORD *)&apb->int_in[4];
				apb->int_out[0] = 0;
				break;

			case WF_BEVENT: /*0x18*/
				if(apb->int_in[2] & 1) {
					win->status |= WIN_UNTOPPABLE;
				}
				else {
					win->status &= ~WIN_UNTOPPABLE;
				};
				
				apb->int_out[0] = 1;
				break;

			default:
				DB_printf("%s: Line %d: Wind_set:\r\n"
								"Unknown mode %d",__FILE__,__LINE__,apb->int_in[1]);
				apb->int_out[0] = 0;
		};
	}
	else
		apb->int_out[0] = 0;
	
	Rdwrs_operation(WENDWRITE);
}


WORD	winoncoord(WORD x,WORD y) {
  WINLIST *l = globals.win_vis;
  
  while(l) {
    if((x >= l->win->totsize.x) &&
       (x < (l->win->totsize.x + l->win->totsize.width)) &&
       (y >= l->win->totsize.y) &&
       (y < (l->win->totsize.y + l->win->totsize.height))) {
      return l->win->id;
    };
    
    l = l->next;
  };
  
  return 0;
}

/*wind_find 0x006a*/
void	Wind_find(AES_PB *apb) {	
	Rdwrs_operation(WSTARTREAD);
	apb->int_out[0] = winoncoord(apb->int_in[0],apb->int_in[1]);
	Rdwrs_operation(WENDREAD);
}

/****************************************************************************
 * Wind_beg_update                                                          *
 *  Begin to update screen.                                                 *
 ****************************************************************************/
WORD                   /* 1 if ok or 0.                                     */
Wind_beg_update(       /*                                                   */
WORD apid)             /* Application to begin update.                      */
/****************************************************************************/
{
	return Rdwrs_get_sem(apid,&update);
}

/****************************************************************************
 * Wind_end_update                                                          *
 *  End update of screen.                                                   *
 ****************************************************************************/
WORD                   /* 1 if ok or 0.                                     */
Wind_end_update(       /*                                                   */
WORD apid)             /* Application to release exclusive right.           */
/****************************************************************************/
{
	return Rdwrs_rel_sem(apid,&update);
}

/*wind_update 0x006b*/
void	Wind_update(AES_PB *apb) {
	switch(apb->int_in[0]) {
	case	BEG_UPDATE + 0x100: /* check & set */
		if ((update.count > 0) && (apb->global[2] != update.apid)) {
			apb->int_out[0] = 0;
			break;
		}

	case	BEG_UPDATE:
		apb->int_out[0] = Wind_beg_update(apb->global[2]);
		break;

	case	END_UPDATE:
		apb->int_out[0] = Wind_end_update(apb->global[2]);
		break;

	case	BEG_MCTRL + 0x100: /* check & set */
		apb->int_out[0] = Evhd_gain_mctrl(apb->global[2],MCTRL_BUTNB);
		break;

	case BEG_MCTRL:
		apb->int_out[0] = Evhd_gain_mctrl(apb->global[2],MCTRL_BUTTON);
		break;

	case END_MCTRL:
		apb->int_out[0] = Evhd_release_mctrl(apb->global[2]);
		break;

	default:
		DB_printf("%s: Line %d: Wind_update:\r\n"
		       "Unknown mode %d",__FILE__,__LINE__,apb->int_in[1]);
		apb->int_out[0] = 0;
	}
}



/*wind_calc 0x006c*/
void	Wind_calc(AES_PB *apb) {
	calcworksize(apb->int_in[1],(RECT *)&apb->int_in[2]
		,(RECT *)&apb->int_out[1],apb->int_in[0]);
	
	apb->int_out[0] = 1;
}
